#!/bin/bash
#
# copyarch - Admin backup copy script
#
# Type  : Utility
#
# Author: Rob Allen
#
# Date  : 29.04.2008
#
# Program Description
# ===================
# Script to restore a Swivel Appliance from a backup.
# Supported scripts: Will only restore from backup.sh
#
#
# Amendment History
# =================
# 09.05.08 Created today (RA)
# 19.09.08 Changed script to use admin user for the scp command (RA)
# 17.02.09 Changed scp destination to work with new SFTP command
# 25.10.09 Check if any files to restore, if not exit script (RA)
# 02.11.09 Added function to extract PINsafe DB and create smaller archive (RA)
# 02.11.09 Changed Exiting routine to return to main listing, not exit (RA)
# 02.11.09 Replaced echo -n with read -p where input required (RA)
# 08.04.10 Added error checking to IP entry field (RA)
# 27.07.10 Added use of nmap, to detect remote scp connection before copy. (RA)
# 11.11.10 Placed "" marks around var TYPE to allow spacing (RA)
# 13.12.10 Changed 'check_ans' to match all other scripts (RA)
# 25.01.12 updated the copyright year as requested by Chris. (RB)
#

PINSAFE_TYPE="$1"
RemotehostIP=$2
LOCK=/var/lock/subsys
BACKUPROOT=/backups
AUDIT_LOG=/var/log/cmi/audit.log
COPYRIGHT_YEAR="2012"
TOMCAT=/usr/local/tomcat
WEBINF=$TOMCAT/webapps/pinsafe/WEB-INF
version="0.11"
inv=`tput smso`
nor=`tput rmso`

if test -d /home/swivel/.swivel; then
 SWIVEL=/home/swivel/.swivel
else
 SWIVEL=$WEBINF
fi

cd $BACKUPROOT

heading() {
 clear
 echo "${inv} Swivel Maintenance (c) $COPYRIGHT_YEAR${nor}                                             ${inv}$PINSAFE_TYPE${nor}"
 if test $DR; then
  if test $DR = "yes"; then
   clear
   echo "${inv} Swivel Maintenance (c) $COPYRIGHT_YEAR${nor}                                                  ${inv}DR${nor}"; fi
 fi
 echo 
 echo $1
 echo
}

s_heading() {
 clear
 echo "${inv} Swivel Maintenance (c) $COPYRIGHT_YEAR${nor}                                             ${inv}$PINSAFE_TYPE${nor}"
 if test $DR; then
  if test $DR = "yes"; then
   clear
   echo "${inv} Swivel Maintenance (c) $COPYRIGHT_YEAR${nor}                                                  ${inv}DR${nor}"; fi
 fi
 echo
}

audit() {
 audit_date=`date "+%d%m%y %T"`
 echo "$audit_date : $1" >> $AUDIT_LOG
}

warning() {
echo "Backup created: $date_run"
echo
echo "   By continuing this process will copy the selected archive: "
echo
echo "   Archive: $archive.tar.gz   ->   Appliance : $dest_ip"
echo
}

yorn() {
 echo
 read -p "Are you sure? (Yes/No): " yn
 check_ans $1 $yn
}

check_ans() {
 if test ! $2; then
  $1; fi
 if test ! $2 = "Yes" && test ! $2 = "yes" && test ! $2 = "y" && test ! $2 = "Y"; then
  $1; fi
}

validate_IP() {
 ERR=0
 IP_ADDR_VAL=$(echo "$1" | grep -Ec '^(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9])\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[0-9])$')

 if test $IP_ADDR_VAL -eq 0; then
  IPs_invalid format $1; fi
}

IPs_invalid() {
 ERR=1
 echo
 if test $1 = "format"; then
  echo "The format of the IP address \"$2\" is incorrect."; fi
 echo; read -p "Press return to Continue" yn
}

completed() {
 echo
 echo "The copy process has now completed."
 echo
 read -p "Press return to Continue." yn
}

create_res_array() {
 for name in `ls -t *.info|grep -v latest`; do
  part1=`echo $name | cut -d\. -f1`
  part2=`echo $name | cut -d\. -f2`
  archive_name="$part1.$part2"
  if test -f $archive_name.tar.gz && test $archive_name.info; then
   echo $archive_name; fi
 done
}

populate_res_array() {
 cd $BACKUPROOT

 archlist=( `create_res_array` )

 count=0
 for name in ${archlist[@]}
 do
  archlist[$count]=$name
  ((count = count + 1))
 done
 totcount=$count
}

disp_backups() {
 heading "Select a Backup archive to copy"
 dispcount=1
 count=1
 fncount=0
 page=1
 choice=""
 for archive_name in ${archlist[@]}
 do
  date_run=`grep ^DATE $archive_name.info |cut -d\= -f2`
  backup_type="`grep ^TYPE $archive_name.info |cut -d\= -f2`"
  if test "$backup_type" = "P"; then
   TYPE_text="Swivel Backup"; fi
  if test "$backup_type" = "P-DB"; then
   TYPE_text="Swivel DB Only"; fi
  if test "$backup_type" = "A"; then
   TYPE_text="Full Backup"; fi

  backup_text=`grep ^COMMENT $archive_name.info |cut -d\= -f2`

  printf "%-3s %-29s %-14s %-15s\n" "$count." "`echo $date_run` :" "$TYPE_text" "($archive_name).tar.gz"

  
  if test $dispcount = 15 && test $count != $totcount; then
   ((page = page + 1))
   dispcount=0
   next_page; fi

  ((dispcount = dispcount + 1))
  ((count = count + 1))
  ((fncount = fncount + 1))
 done

 if test ! $choice; then
  select_q " Exit"
  if test $choice; then
   select_choice
   main
   choice=""; fi
 fi
 exit
}

select_q() {
 echo "0. $1"
 echo
 read -p "Select: " choice
}

next_page() {
 select_q "Next Page"

 if test ! $choice; then
  heading "Select a backup archive to copy (page: $page)."
 else
  if test $choice = 0; then
   heading "Select a backup archive to copy (page: $page)."
   choice=""
  else
   select_choice
   main
   choice=""
   break
  fi
 fi
}

select_choice() {
 if test $choice -lt 0 || test $choice -gt $totcount; then
  echo "Not a valid option, not that many backups"
  start; fi
 
 if test $choice -eq 0; then
  echo "Finished"
  exit; fi

 ((choice = choice - 1))
 src_dir=${archlist[$choice]}
}

select_dest() {
 heading "Destination IP address"
 echo "Enter an IP address to copy the archive too."
 echo
 if test $RemotehostIP; then
  read -p "Destination IP of the remote appliance ($RemotehostIP) : " dest_ip
  if test ! $dest_ip; then
   dest_ip=$RemotehostIP; fi
 else
  read -p "Destination IP address : " dest_ip
  if test ! $dest_ip; then
   echo; echo -n "The destination IP cannot be blank"
   sleep 2
   start; fi
 fi
 validate_IP $dest_ip
 if test $ERR = "1"; then
  start; fi
}

copy_arch() {
 nmap -P0 -n -p 22 $dest_ip | grep open > /dev/null
 if test $? = 0; then
  scp $BACKUPROOT/$archive.* admin@$dest_ip:/backups/restore
  if test $? = 0; then
   completed
  else
   echo "Unable to copy the archive"
   echo; read -p "Press return to Continue." yn
  fi
 else
  echo "Unable to connect to the specified server: $dest_ip"
  echo; read -p "Press return to Continue." yn
 fi
}

check_dir() {
 cd $BACKUPROOT
 if ! `ls *.info > /dev/null 2>&1`; then
  echo "No files available to copy"
  echo; read -p "Press return to exit" yn
  exit; fi
}

extract_db() {
 heading "Extract Swivel DB from archive"
 echo "The selected archive contains a Swivel DB."
 echo
 echo "Copying just the Swivel DB will be faster over a WAN"
 echo
 read -p "Do you wish to extract the Swivel DB? (Yes/No): " yn
 if test $yn; then
  if test $yn = "Yes" || test $yn = "yes"; then
   extract_db_to_dir
  elif test $yn = "No" || test $yn = "no"; then
   echo
  else
  extract_db; fi
 else
  extract_db
 fi
}

extract_db_to_dir() {
 cd $BACKUPROOT
 tar xfz $archive.tar.gz
 dir_name=`date +%d%m%y.$$`
 BACKUP_DIR=$BACKUPROOT/$dir_name
 mkdir -p $BACKUP_DIR
 cd $archive

 echo
 echo "Extracting Swivel $DB_IN_USE DB from archive and creating new archive"
 if test $DB_IN_USE = "Internal"; then
  cp -p --parents swivel/db $BACKUP_DIR; fi
 if test $DB_IN_USE = "MySQL"; then
  cp -p --parents mysql/pinsafe_backup.sql $BACKUP_DIR; fi

 cp -p --parents swivel/conf/config.xml $BACKUP_DIR

 echo "DATE=`date`" > $BACKUP_DIR.info
 echo "TYPE=P-DB" >> $BACKUP_DIR.info
 echo "COMMENT=Swivel DB extracted from $archive archive" >> $BACKUP_DIR.info
 cd /backups; tar cfz `echo $dir_name`.tar.gz $dir_name
 echo "Swivel DB extracted from archive and new archive created"
 echo "New archive name: $dir_name"
 rm -rf $dir_name $archive
 echo; read -p "Press return to Continue." yn
 choice=""
 start
}

db_used() {
 CONFIG_XML=$BACKUPROOT/$archive/swivel/conf/config.xml
 XML="<option generated=\"true\" selected=\"true\""

 DB_IN_USE=External
 if test -f $CONFIG_XML; then
  if grep "$XML>Shipping</option>" $CONFIG_XML > /dev/null; then
   DB_IN_USE=Shipping; fi
  if grep "$XML>Internal</option>" $CONFIG_XML > /dev/null; then
   DB_IN_USE=Internal; fi
  if grep "$XML>MySQL 5</option>" $CONFIG_XML > /dev/null; then
   DB_IN_USE=MySQL; fi
 fi
}

chk_db_included() {
 cd $BACKUPROOT
 tar -zxf $archive.tar.gz $archive/swivel/conf/config.xml
 db_used
 rm -rf $archive
}

chk_backup_type() {
 cd $BACKUPROOT
 backup_type="`grep ^TYPE $archive.info |cut -d\= -f2`"
}

main() {
 archive=${archlist[$choice]}
 cd $BACKUPROOT
 chk_backup_type
 if test "$backup_type" = "A" || test "$backup_type" = "P"; then
  chk_db_included; fi
 if test $DB_IN_USE = "Internal" || test $DB_IN_USE = "MySQL"; then
  extract_db; fi
 s_heading
 select_dest
 s_heading
 warning
 yorn start
 audit "Backup file copied to $dest_ip"
 s_heading
 warning
 copy_arch
}

start() {
 heading "Copy a backup"
 check_dir
 populate_res_array
 disp_backups
}

trap start 2
start
